/*   FILE: output.vala -- Handle output either to a PiFace CAD or stdout.
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 22 February 2018
 *
 * Copyright (c) 2018 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

class Output : GLib.Object {
	static uint output_shift = 0;
        static string output_str = "";

	public Output() {
		if (! no_pifacecad) {
			GLib.Timeout.add (750, render_output);
		}
	}

	private bool render_output () {
		Pifacecad.lcd_clear ();

		if (output_str.length <= 15) {
			Pifacecad.lcd_write (output_str);
		} else {
			var builder = new StringBuilder ();
			uint rest = output_str.length > 30 ? 15 : output_str.length - 15;
			uint cutoff = output_str.length > 30 ? output_str.length - 30 : 0;

			// FIXME: Should be able to replace 15 with 16,
			// but libpifacecad seems to require a "cell" for '\n'.
			// See https://github.com/piface/libpifacecad/issues/2.
			builder.append (output_str.substring (output_shift, 15));
			builder.append ("\n ");
			builder.append (output_str.substring (15 + output_shift, rest));

			Pifacecad.lcd_write (builder.str);

			output_shift = output_shift < cutoff ? output_shift + 1 : 0;
		}

		return true;
	}

	public void output (string str) {
		GLib.stdout.printf ("%s\n", str);
		output_shift = 0;
		output_str = str;
		if (! no_pifacecad) {
			render_output(); /* Also registered as GLib.Timeout function for long-string animation. */
		}
	}
}
